﻿package com.alecdebusschere.src{
	import flash.display.*;
	import flash.text.*;
	
	import com.alecdebusschere.src.drawUtils.*;
	import flash.geom.*;
	import fl.transitions.Tween;
	import fl.transitions.easing.*;
	
	internal class Structure extends Sprite{
		
		//external data
		internal var xmlWorks:XML, xmlCategories:XML;
		
		//internal data
		internal var works:Array = new Array();//list all the works
		internal var relations:Array = new Array();//pour les liens "cousins"
		internal var tree:Array;//root works (each work has a children parameter)
		internal var categories:Array;
		
		
		//geom data
		internal var diameter:int, levelDiameter:int, nblevels:int = 1, nbleaves:int = 0, leaveNode = 0;
		internal var scalePoint:Point = new Point(0, 0);
		
		//display
		internal var displayState:int;
		internal var mapDisplay:Sprite = new Sprite();
		internal var treeDisplay:Sprite = new Sprite();
		internal var worksDisplay:Sprite = new Sprite();
		
		//motion
		private var tweens:Array = new Array();
		
		var mainStage:Stage;
		
		public function Structure(xmlCategories:XML, xmlWorks:XML, mainStage:Stage){
			this.xmlCategories = xmlCategories;
			this.xmlWorks = xmlWorks;
			this.mainStage = mainStage;
			
			categories = createCategories();
			
			tree = createWorks(this.xmlWorks);
			
			render();
			
			this.scaleX = Settings.startRatio;
			this.scaleY = Settings.startRatio;
			
			//displayTree(tree);
		}
		
		
///////////////////////ABSTRACT
		private function createCategories(){
			var categories:Array = new Array();
			
			for each(var cat:XML in this.xmlCategories..cat)
				categories.push(new Category(cat.@id, cat.@title, cat.@color));
				
			return categories;
		}
		
		//create Work objects.
		private function createWorks(workList:XML, level = 1, parent:Work = null){
			
			var children:Array = new Array();
			
			for each(var work:XML in workList.child("work")){
				//new object work
				var w = new Work(work.@id, work.@title, level, parent);
				
				this.works.push(w);
				
				//set up relations between works
				setRelations(w, work.@related);
				
				//set up categories pointers for each work
				setCatPtr(w, work.@categories);
				
				//let's get recursive to set up the hierarchy
				if(work.child("work").length() > 0)
					w.setChildren(createWorks(work, level+1, w));
				else{
					if(level > nblevels) nblevels = level;
					this.nbleaves++;
				}
				
				children.push(w);
			}
			return children;
		}
		
		private function setRelations(w:Work, relString:String){
			var idsRelWorks = relString.split("-");
			for each(var id:int in idsRelWorks){ 
				for each(var work:Work in works){
					if(work.id == id)
						relations.push(new Array(w, work));
				}
			}
		}
		
		private function setCatPtr(w:Work, catString:String){
			var idsCat = catString.split("-");
			for each(var id:int in idsCat){
				for each(var cat:Category in categories){
					if(cat.id == id) w.categories.push(cat);
				}
			}
		}
		//debug utils
		private function displayTree(works:Array){//testing the structure
			for(var i:int = 0; i < works.length; i++){
				if(works[i].children.length > 0)
					displayTree(works[i].children);
				if(!works[i].root) trace("parent" + works[i].parent.workTitle);
				
				trace(works[i].workTitle);
			}
		}
///////////////////////END ABSTRACT
///////////////////////GRAPHIC
		internal function render(){
			
			
			
			drawMap();
			
			
			drawWorks(this.tree);
			
			
			drawRelations();
			drawTree(this.tree);
			
			addChild(this.mapDisplay);
			addChild(this.treeDisplay);
			addChild(this.worksDisplay);
		}
		private function drawMap(){
			
			
			this.diameter = nbleaves * nblevels * Settings.wDiameter*Settings.ratioDiameter;
			this.mapDisplay.graphics.beginFill(0xFFFFFF);
			this.mapDisplay.graphics.drawRect(-this.diameter/2-Settings.scExternalDist, -this.diameter/2-Settings.scExternalDist, this.diameter+Settings.scExternalDist*2, this.diameter+Settings.scExternalDist*2);
			this.levelDiameter = this.diameter/nblevels;
			//trace(nbleaves + " * " + nblevels + " * " + Settings.wDiameter + " = " +this.diameter);
			
			
			//trace(nblevels);
			for(var i:int = nblevels; i >= 1; i--){
				if(i == 1){
					var circle:Shape = new Shape();
					circle.graphics.lineStyle(Settings.sStrokeWeight, Settings.sStrokeColor, Settings.sStrokeAlpha);
					circle.graphics.drawCircle(0,0,this.levelDiameter/2*i);
					this.mapDisplay.addChild(circle);
				}else{
					
					var dashedCircle = new DashedCircle(this.levelDiameter/2*i, Settings.sStrokeWeight, Settings.sDashLength, 
														Settings.sStrokeColor, Settings.sStrokeAlpha, Settings.sBackground, 
														Settings.sBackgroundColor, Settings.sBackgroundAlpha);
					this.mapDisplay.addChild(dashedCircle);
				}
			}
			
			
			
		}
		
		private function drawWorks(works:Array){
			
			for(var i:int = 0; i < works.length; i++){
				var totalAngle = 0;
				var nbChildren = 0;
				
				if(works[i].children.length > 0){
					
					drawWorks(works[i].children);
					//placer les parents
					nbChildren = works[i].children.length;
					
					for each(var child:Work in works[i].children){
						//totalAngle += 180* child.angle/Math.PI;
						totalAngle += child.angle;
					}
					
					works[i].setPos(this.levelDiameter, 180*(totalAngle/nbChildren)/Math.PI);
					
					
					
				}else{
					
					this.leaveNode++;
					works[i].setPos(this.levelDiameter, 360/nbleaves*leaveNode);
					totalAngle = works[i].angle;
					nbChildren = 1;
					//trace(works[i].angle);
				}
				
				//calculation lines
				var cLines:Shape = new Shape();
				cLines.graphics.lineStyle(Settings.scStrokeWeight, Settings.scStrokeColor, Settings.scStrokeAlpha);
				cLines.graphics.moveTo(Math.cos(works[i].angle)*this.levelDiameter/2, Math.sin(works[i].angle)*this.levelDiameter/2);
				cLines.graphics.lineTo(Math.cos(works[i].angle)*(this.diameter/2+Settings.scExternalDist), 
										 Math.sin(works[i].angle)*(this.diameter/2+Settings.scExternalDist));
				this.mapDisplay.addChild(cLines);
				
				//calculation text
				var indicationContener:Sprite = new Sprite();
				var indication:TextField = new TextField();
				indication.embedFonts = true;
				indication.autoSize = TextFieldAutoSize.LEFT;
          		//indication.antiAliasType = AntiAliasType.ADVANCED;
          		indication.defaultTextFormat = Settings.formatIndic;
				indication.text = 180*(totalAngle/nbChildren)/Math.PI+"°";
				indication.rotation = 180*(totalAngle/nbChildren)/Math.PI;
				indicationContener.addChild(indication);
				
				indicationContener.x = Math.cos(works[i].angle)*(this.diameter/2+Settings.scExternalDist-indication.width);
				indicationContener.y = Math.sin(works[i].angle)*(this.diameter/2+Settings.scExternalDist-indication.width);
				
				
				
				this.mapDisplay.addChild(indicationContener);
				
				works[i].drawWork();
				worksDisplay.addChild(works[i]);
			}
		}
		
		private function drawTree(works:Array){
			
			for(var i:int = 0; i < works.length; i++){
				if(works[i].children.length > 0){
					drawTree(works[i].children);
					
				}
				if(works[i].rootW) continue;
				
				//var dist:Number = this.levelDiameter/2*(works[i].level-1);
				
				var dist:Number = Math.sqrt(works[i].pos.x*works[i].pos.x + works[i].pos.y*works[i].pos.y)/2;
				this.treeDisplay.graphics.lineStyle(Settings.stStrokeWeight, Settings.stStrokeColor, Settings.stStrokeAlpha);
				
				this.treeDisplay.graphics.moveTo(works[i].pos.x, works[i].pos.y);
				if(works[i].angle != works[i].parentW.angle){
					this.treeDisplay.graphics.curveTo(Math.cos(works[i].angle)* dist, 
												  	  Math.sin(works[i].angle)* dist,
												  	  works[i].parentW.pos.x, works[i].parentW.pos.y);
				}else{
					this.treeDisplay.graphics.lineTo(works[i].parentW.pos.x, works[i].parentW.pos.y);
				}
				
				this.treeDisplay.graphics.lineStyle(Settings.st2StrokeWeight, Settings.st2StrokeColor, Settings.st2StrokeAlpha);
				
				this.treeDisplay.graphics.moveTo(works[i].pos.x, works[i].pos.y);
				if(works[i].angle != works[i].parentW.angle){
					this.treeDisplay.graphics.curveTo(Math.cos(works[i].angle)* dist, 
												  	  Math.sin(works[i].angle)* dist,
												  	  works[i].parentW.pos.x, works[i].parentW.pos.y);
				}else{
					this.treeDisplay.graphics.lineTo(works[i].parentW.pos.x, works[i].parentW.pos.y);
				}
				
			}
			
		}
		
		private function drawRelations(){
			this.treeDisplay.graphics.lineStyle(Settings.srStrokeWeight, Settings.srStrokeColor, Settings.srStrokeAlpha);
			for each(var relation:Array in this.relations){
				this.treeDisplay.graphics.moveTo(relation[0].pos.x, relation[0].pos.y);
				this.treeDisplay.graphics.lineTo(relation[1].pos.x, relation[1].pos.y);
			}
		}
///////////////////////////////////END GRAPHIC
///////////////////////////////////MOTION
		internal function resetTweens(){
			for each(var tween:Tween in this.tweens){
				tween.stop();
			}
			this.tweens = new Array();
		}
		internal function translateBy(byX:Number, byY:Number){
			
			if(this.x + byX > this.mainStage.stageWidth+this.width/2 - Settings.navigMove*this.scaleX){
				byX = this.mainStage.stageWidth + this.width/2 - this.x - Settings.navigMove*this.scaleX;
			}
			else if(this.x + byX < 0-this.width/2 + Settings.navigMove*this.scaleX){
				byX = -this.width/2-this.x+Settings.navigMove*this.scaleX;
			}
			if(this.y + byY > this.mainStage.stageHeight+this.height/2 - Settings.navigMove*this.scaleY){
				byY = this.mainStage.stageHeight + this.height/2 - this.y - Settings.navigMove*this.scaleY;
			}
			else if(this.y + byY < 0-this.height/2 + Settings.navigMove*this.scaleY){
				byY = -this.height/2-this.y+Settings.navigMove*this.scaleY;
			}
			if(byX != 0)
				this.tweens.push(new Tween(this, "x", Strong.easeOut, this.x, this.x+byX, 1, true));
			
			if(byY != 0)
				this.tweens.push(new Tween(this, "y", Strong.easeOut, this.y, this.y+byY, 1, true));
			
			

		}
		internal function translateTo(toX:Number, toY:Number){
			
			this.tweens.push(new Tween(this, "x", Strong.easeOut, this.x, toX, 1, true));
			this.tweens.push(new Tween(this, "y", Strong.easeOut, this.y, toY, 1, true));
			
			

		}
		internal function zoomBy(zoomRatio:Number){
			this.tweens.push(new Tween(this, "scaleX", Strong.easeOut, this.scaleX, this.scaleX+zoomRatio, 1, true));
			this.tweens.push(new Tween(this, "scaleY", Strong.easeOut, this.scaleY, this.scaleY+zoomRatio, 1, true));
			
		}
		internal function zoomTo(zoomRatio:Number, stageX:Number, stageY:Number){
			var moveX = stageX - scalePoint.x * zoomRatio;
			var moveY = stageY - scalePoint.y * zoomRatio;
						
			translateTo(moveX, moveY);
			
			this.tweens.push(new Tween(this, "scaleX", Strong.easeOut, this.scaleX, zoomRatio, 1, true));
			this.tweens.push(new Tween(this, "scaleY", Strong.easeOut, this.scaleY, zoomRatio, 1, true));
			
		}
		/*internal function gotoWork(w:Object, stageW:Number, stageH:Number){
			translateTo(stageW/2-w.pos.x*Settings.maxRatio, stageH/2-w.pos.y*Settings.maxRatio);
			 zoomTo(Settings.maxRatio);
			
		}*/
		
		
		
		
		
	}
}